TAG_DOCUMENT = 'w:document'
TAG_BODY = 'w:body'
TAG_TBL = 'w:tbl'
TAG_TR = 'w:tr'
TAG_HDR = 'w:hdr'
TAG_FTR = 'w:ftr'
TAG_P = 'w:p'
TAG_TABLE = 'w:tbl'
TAG_TABLE_CELL = 'w:tc'
TAG_TABLE_ROW = 'w:tr'
TAG_R = 'w:r'
TAG_RPR = 'w:rPr'
TAG_T = 'w:t'
TAG_SECT_PR = 'w:sectPr'
TAG_P_STYLE = 'w:pStyle'
TAG_R_STYLE = 'w:rStyle'
ATTR_VAL = 'w:val'
ATTR_COLOR = 'w:color'
ATTR_FILL = 'w:fill'
ATTR_ID = 'r:id'
ATTR_WIDTH = 'w:w'
VAL_TYPE_DXA = 'dxa'
ATTR_TYPE = 'w:type'
TAG_PPR = 'w:pPr'
TAG_COLOR = 'w:color'
TAG_SHADOW = 'w:shd'
TAG_ALIGN = 'w:jc'

ATTR_VAL_CLEAR = "clear"
ATTR_VAL_AUTO = "auto"

TAG_BOLD = 'w:b'
TAG_BOLD_X = 'w:bCs'
TAG_ITALIC = 'w:i'
TAG_ITALIC_X = 'w:iCs'
TAG_UNDERLINE = 'w:u'
ATTR_VAL_UNDERLINE = 'single'
TAG_STRIKE = 'w:strike'
TAG_HYPERLINK = 'w:hyperlink'

TAG_SOLID_FILL = 'a:solidFill'
TAG_NO_FILL = 'a:noFill'
TAG_SRGB_CLR = 'a:srgbClr'
